/* AVR Simon [avrsimon]
 *
 * main.c
 *
 * Created by Joe Grand, Grand Idea Studio, Inc.
 * http://www.grandideastudio.com
 *
 * This file contains code for the Atmel ATTiny2313V using avr-gcc 4.3.3.
 *  
 * Distributed under a Creative Commons Attribution-Share Alike 3.0 Unported license:
 * http://creativecommons.org/licenses/by-sa/3.0/
 *
 * Change log:
 *
 * Version 1.0, Initial release, October 26, 2009
 
 * Version 1.1, September 7, 2010
 * Changes (primarily to allow code to compile under avr-gcc version 4):
 * - Removed PROGMEM macro from pointer declarations
 * - Forced no-inlining of delay_ms() to save code space
 * - Added (unsigned int) cast to eeprom_write_byte and eeprom_read_byte calls
 * 
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include "avrsimon.h"

void delay_ms(unsigned int ms) __attribute__((noinline)); 

unsigned char  game_config;      // game configuration flags
unsigned char  move_number;      // move number (also address of internal EEPROM move location)
unsigned char  current_move;     // current move (same as EEPROM value so we only have to read each move from EEPROM once)
unsigned char  random_byte;	  	 // byte value used for pseudo-random number generator 
volatile unsigned char  timeout_count;    // timeout delay limit (set to volatile because it is being incremented by timer1 interrupt but used in our main program)

/*******************************************************************************************************************************************************/
/* tone generation based on xyloduino code (http://www.rocketnumbernine.com/2009/03/27/xyloduino-simple-arduinopiezo-organ/) and heavily modified      */

 // prescale for octaves
const unsigned char pre[] = {5, 5, 4, 4, 3, 3};

#define STARTGAME_SND_SIZE 7
const struct note_struct startgame_snd[STARTGAME_SND_SIZE] PROGMEM = {{4, D_note, 200}, {3, B_note, 200}, {4, D_note, 100}, {3, B_note, 100}, {4, E_note, 100}, {4, D_note, 200}, {3, A_note, 200}};
PGM_P startgame_snd_p = (void*)startgame_snd; 

#define GAMEOVER_SND_SIZE 10
const struct note_struct gameover_snd[GAMEOVER_SND_SIZE] PROGMEM = {{3, F_note, 400}, {3, F_note, 75}, {3, F_note, 100}, {3, F_note, 500}, {3, A_note, 100}, {3, G_note, 200}, {3, F_note, 100}, {3, F_note, 200}, {3, E_note, 100}, {3, F_note, 400}};
PGM_P gameover_snd_p = (void*)gameover_snd; 

// Original Simon tones (courtesy of R.H. Baer, US Patent #4,207,087) 
// Tone 1, blue, 392Hz (G)
// Tone 2, yellow, 330Hz (E)
// Tone 3, red, 262Hz (C)
// Tone 4, green, 196Hz (G)
// Fail/rasp, 67ms @ 0.3ms pulse (unused in this project)
const struct note_struct blue_snd[1] PROGMEM = {{3, G_note, BETWEEN_MOVE_ON_DELAY}};
PGM_P blue_snd_p = (void*)blue_snd; 

const struct note_struct yellow_snd[1] PROGMEM = {{3, E_note, BETWEEN_MOVE_ON_DELAY}};
PGM_P yellow_snd_p = (void*)yellow_snd; 

const struct note_struct red_snd[1] PROGMEM = {{3, C_note, BETWEEN_MOVE_ON_DELAY}};
PGM_P red_snd_p = (void*)red_snd; 

const struct note_struct green_snd[1] PROGMEM = {{2, G_note, BETWEEN_MOVE_ON_DELAY}};
PGM_P green_snd_p = (void*)green_snd; 

/*******************************************************************************************************************************************************/

// interrupt handler
SIGNAL(TIMER1_COMPA_vect)	// timer1, set to interrupt every 1 second
{
	++timeout_count;	// increase our global variable
}

EMPTY_INTERRUPT(PCINT_vect)   // pin change interrupt request, use this to wake up the device on keypress
				
				
int main(void)
{
	hardware_init();	// configure hardware to a known state on power-up
    simon_config();		// set game configuration as specified from buttons on power-up
	
	while(1)			// play the game forever
		simon_game();
	
    return(0);  // we should never get here!
}


int simon_game(void)
{
	unsigned char i;
	
	// prepare to go to sleep/idle mode...
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	cli();
	sleep_enable();
	sei();
	sleep_cpu(); // go to sleep here to save power
	
	// zzz...
	
	sleep_disable(); // wake up when a button is pressed to start the game
	
    // seed random number generator with current value of timer1 and whatever button was pushed (to help randomize the seed - it works good enough!)
    random_byte = (char)(TCNT1 + SW_PIN);
	
	// reset game-specific variables
    move_number = 0;
    timeout_count = 0;
	
	all_led_off(); // make sure all LEDs are off

	while (1)
    {
		_cbi(TIMSK, OCIE1A);		// disable timer 1 interrupt
		delay_ms(1000);				// allow the player to prepare before each level starts
		simon_play_moves();			// let the cpu make the sequence of moves
		
		_sbi(TIMSK, OCIE1A);		// enable the output compare A match interrupt (timer 1)
		i = simon_read_moves();		// let the player respond
		_cbi(TIMSK, OCIE1A);		// disable timer 1 interrupt

		if (i)	// if bad move or timeout...
        {
            simon_failed_input();	//... then end the game
            return(1);
        }
		
        // if we get here, the player's move must have been successful
        ++move_number;	// increment the move counter
        if (move_number == EEPROM_SIZE) // if the player has achieved the maximum number of moves possible in a single game (limited by the size of the internal EEPROM), then they win!
        {								// this would be a most impressive feat! i wonder if there is a guiness book of world records entry for the most number of moves in a simon game?
			delay_ms(250);
          			
			// turn all LEDs on in celebration :)
			_ledon(LED_RED);
			_ledon(LED_GREEN);
			_ledon(LED_YELLOW);
			_ledon(LED_BLUE);	
  			
            return(1);   // loop back to main routine and wait for a keypress to start a new game
        }
    }
	
	return(1);
}


void simon_play_moves(void)
{
	unsigned char i;
	
    current_move = rand() & 0x03; // pull a number from the PRNG to create the next move (can only be one of four values)
	eeprom_write_byte((unsigned char*)(unsigned int)move_number, current_move);  // store current move in internal EEPROM and add to the end of the sequence
		
    for (i = 0; i <= move_number; ++i)  // play all moves for the current level
    {
        current_move = eeprom_read_byte((unsigned char *)(unsigned int)i); // retrieve the current move from the internal eeprom
		
        switch (current_move)  // perform the move depending on color
        {
            case RED_MOVE:
				red_move();
                break;
			case GREEN_MOVE:
				green_move();
                break;
			case BLUE_MOVE:
				blue_move();
                break;
			case YELLOW_MOVE:
				yellow_move();
                break;
        }
				
		all_led_off(); // turn off LEDs
				
        // delay to wait before showing the next move
        if (game_config & EXPERT_MODE)				// if expert mode was selected... 
            delay_ms(BETWEEN_MOVE_OFF_DELAY_HARD);	// ...then have a shorter delay
        else
            delay_ms(BETWEEN_MOVE_OFF_DELAY_EASY);  // otherwise the player gets the easy/longer delay
    }	
}


int simon_read_moves(void)
{
	unsigned char i;
	unsigned char but, timeout;
	
    if (game_config & EXPERT_MODE)  // if expert mode was selected...
        timeout = TIMEOUT_DELAY_HARD; // ...then set the speed accordingly
    else
        timeout = TIMEOUT_DELAY_EASY; // in easy mode, the player has a longer time to respond to each move in the sequence
	
    for (i = 0; i <= move_number; ++i)  // read each move stored in the EEPROM and compare it against the player's input
    {
        if (game_config & REVERSE_MODE) // if reverse mode was selected...
            current_move = eeprom_read_byte((unsigned char *)(unsigned int)(move_number - i));  // reverse the pointers so the last move needs to be entered by the player first (moving backwards through the EEPROM address space)
        else
			current_move = eeprom_read_byte((unsigned char *)(unsigned int)i);
		
        timeout_count = 0;
		
        while (timeout_count != timeout)  // wait for player to replay the moves in the sequence
        {
			if ((SW_PIN & SW_NO_BUTTON) != SW_NO_BUTTON) // if a button is being pressed...
            {
                delay_ms(25);					// ...then do poor man's debounce
				but = ~(SW_PIN & SW_NO_BUTTON); // and read the values again
				
                // if correct, play sound and show LED of current button press
                if ((but & SW_RED_MASK) && (current_move == RED_MOVE))
                {
                    timeout_count = 0;
                    red_move();
                    while(!(SW_PIN & SW_RED_MASK))    // wait until player releases button
					{		
						if (timeout_count >= timeout) return(1); // prevent player from holding down button for too long
                    }
					goto next; // using gotos in C can sometimes be risky and create hard-to-follow code, but here they are very useful
                }
                else if ((but & SW_GREEN_MASK) && (current_move == GREEN_MOVE))
                {
                    timeout_count = 0;
                    green_move();
                    while(!(SW_PIN & SW_GREEN_MASK))    // wait until player releases button
					{		
						if (timeout_count >= timeout) return(1); // prevent player from holding down button for too long
                    }
					goto next;
                }
                else if ((but & SW_BLUE_MASK) && (current_move == BLUE_MOVE))
                {
                    timeout_count = 0;
                    blue_move();
                    while(!(SW_PIN & SW_BLUE_MASK))    // wait until player releases button
                    {	
						if (timeout_count >= timeout) return(1); // prevent player from holding down button for too long
                    }
					goto next;
                }
                else if ((but & SW_YELLOW_MASK) && (current_move == YELLOW_MOVE))
                {
                    timeout_count = 0;
                    yellow_move();
                    while(!(SW_PIN & SW_YELLOW_MASK))    // wait until player releases button
                    {	
						if (timeout_count >= timeout) return(1); // prevent player from holding down button for too long
                    }
					goto next;
                }
                else
                    return(1);
            }
        }
        return(1); // if we get here, the user has timed out
		
        // code will jump to here if a correct button press was made, so continue onto the next move
	next:
		all_led_off();	// turn off LEDs
    }
	
    return(0);
}


void red_move(void)
{
	if (!(game_config & BLIND_MODE))
		_ledon(LED_RED);
	if (!(game_config & SILENT_MODE))
		play_note(pgm_read_byte(red_snd_p), pgm_read_byte(red_snd_p + 1), pgm_read_word(red_snd_p + 2)); // octave, note, length
	else
		delay_ms(BETWEEN_MOVE_ON_DELAY);  // if silent mode enabled, need to force an on-time delay (instead of relying on play_note)
}


void green_move(void)
{
	if (!(game_config & BLIND_MODE))
		_ledon(LED_GREEN);
	if (!(game_config & SILENT_MODE))
		play_note(pgm_read_byte(green_snd_p), pgm_read_byte(green_snd_p + 1), pgm_read_word(green_snd_p + 2)); // octave, note, length
	else
		delay_ms(BETWEEN_MOVE_ON_DELAY);  // if silent mode enabled, need to force an on-time delay (instead of relying on play_note)	
}


void yellow_move(void)
{
	if (!(game_config & BLIND_MODE))
		_ledon(LED_YELLOW);
	if (!(game_config & SILENT_MODE))
		play_note(pgm_read_byte(yellow_snd_p), pgm_read_byte(yellow_snd_p + 1), pgm_read_word(yellow_snd_p + 2)); // octave, note, length	
	else
		delay_ms(BETWEEN_MOVE_ON_DELAY);  // if silent mode enabled, need to force an on-time delay (instead of relying on play_note)		
}


void blue_move(void)
{
	if (!(game_config & BLIND_MODE))
		_ledon(LED_BLUE);
	if (!(game_config & SILENT_MODE))
		play_note(pgm_read_byte(blue_snd_p), pgm_read_byte(blue_snd_p + 1), pgm_read_word(blue_snd_p + 2)); // octave, note, length		
	else
		delay_ms(BETWEEN_MOVE_ON_DELAY);  // if silent mode enabled, need to force an on-time delay (instead of relying on play_note)	
}


void simon_failed_input(void)  // the player has lost, so end the game...
{
    unsigned char i;
    unsigned char hundreds, tens, ones;
	
	all_led_off();
    switch (current_move) // display the color that was next in the sequence and supposed to be entered by the player
    {
        case RED_MOVE:
			_ledon(LED_RED);
            break;
        case GREEN_MOVE:
			_ledon(LED_GREEN);
            break;
        case BLUE_MOVE:
			_ledon(LED_BLUE);	
            break;
        case YELLOW_MOVE:
			_ledon(LED_YELLOW);
            break;
    }
	
    if (!(game_config & SILENT_MODE)) // if silent mode not selected, play the "game over" music
    {
		for (i = 0; i < GAMEOVER_SND_SIZE * 4; i = i + 4)
		{
			play_note(pgm_read_byte(gameover_snd_p + i), pgm_read_byte(gameover_snd_p + i + 1), pgm_read_word(gameover_snd_p + i + 2)); // octave, note, length		
			delay_ms(60); // pause in between notes
		}
    }
	else
		delay_ms(2750); // pause to let the player see the LED if music isn't playing
	
	all_led_off();	// Turn off LEDs
	
    delay_ms(1500); // pause before displaying the score
	
    hundreds = move_number / 100;
    tens = (move_number % 100) / 10;
    ones = (move_number % 10);
	
    // blink the score on the LEDs using green (hundreds), red (tens), and blue (ones) LEDs
    for (i = 0; i < hundreds; ++i)
    {
		_ledon(LED_GREEN);
        
		if (!(game_config & SILENT_MODE))
			play_note(pgm_read_byte(green_snd_p), pgm_read_byte(green_snd_p + 1), pgm_read_word(green_snd_p + 2)); // octave, note, length		
        else
            delay_ms(BETWEEN_MOVE_ON_DELAY);
		
		_ledoff(LED_GREEN);
        delay_ms(BETWEEN_MOVE_OFF_DELAY_EASY);
    }
	
    for (i = 0; i < tens; ++i)
    {
		_ledon(LED_RED);

        if (!(game_config & SILENT_MODE))
			play_note(pgm_read_byte(red_snd_p), pgm_read_byte(red_snd_p + 1), pgm_read_word(red_snd_p + 2)); // octave, note, length		
        else
            delay_ms(BETWEEN_MOVE_ON_DELAY);
		
		_ledoff(LED_RED);
        delay_ms(BETWEEN_MOVE_OFF_DELAY_EASY);
    }
	
    for (i = 0; i < ones; ++i)
    {
		_ledon(LED_BLUE);	
				
        if (!(game_config & SILENT_MODE))
			play_note(pgm_read_byte(blue_snd_p), pgm_read_byte(blue_snd_p + 1), pgm_read_word(blue_snd_p + 2)); // octave, note, length		
        else
            delay_ms(BETWEEN_MOVE_ON_DELAY);
		
		_ledoff(LED_BLUE);	
        delay_ms(BETWEEN_MOVE_OFF_DELAY_EASY);
    }	
}


void simon_config(void)     // set game configuration as specified from buttons on power-up
{
    unsigned char i;
	
	i = ~(SW_PIN & SW_NO_BUTTON);	// get button values

    // red: silent mode (no tone)
    // green: blind mode (no LEDs)
    // blue: expert mode (faster speed)
    // yellow: reverse mode
    game_config = 0;
    if (i & SW_RED_MASK)
        game_config |= SILENT_MODE;
    if (i & SW_GREEN_MASK)
        game_config |= BLIND_MODE;
    if (i & SW_BLUE_MASK)
        game_config |= EXPERT_MODE;
    if (i & SW_YELLOW_MASK)
        game_config |= REVERSE_MODE;
	
    if (!(game_config & SILENT_MODE)) // if silent mode not selected
    {
		for (i = 0; i < STARTGAME_SND_SIZE * 4; i = i + 4)
        {
			play_note(pgm_read_byte(startgame_snd_p + i), pgm_read_byte(startgame_snd_p + i + 1), pgm_read_word(startgame_snd_p + i + 2)); // octave, note, length		
			delay_ms(50); // pause in between notes
        }
    }
	else
		delay_ms(1000); // delay to give the user a chance to remove their finger from the button
	
	while ((SW_PIN & SW_NO_BUTTON) != SW_NO_BUTTON){}; // wait until no buttons are being pressed
	delay_ms(250); // poor man's debounce
}


void play_note(unsigned char octave, unsigned char note, unsigned int length) // play a note
{
	if (note)
	{
		TCCR0B = pre[(int)octave];  // set the prescaler depending on what octave is selected
		OCR0A = note >> (octave%2); // there are two octaves for each prescale setting, so adjust accordingly
	}
	
	delay_ms(length); // play sound for specified length

	TCCR0B = 0; // turn off sound
}


void hardware_init(void)	// configure hardware to a known state on power-up
{
	// set LED pins to outputs
	_sbi(LED_DDR, LED_RED);
	_sbi(LED_DDR, LED_GREEN);
	_sbi(LED_DDR, LED_YELLOW);
	_sbi(LED_DDR, LED_BLUE);
	
	// turn off LEDs
	all_led_off();
	
	// set switch pins to inputs
	_cbi(SW_DDR, SW_RED);
	_cbi(SW_DDR, SW_GREEN);
	_cbi(SW_DDR, SW_YELLOW);
	_cbi(SW_DDR, SW_BLUE);
	
	// enable internal pull-up resistors for switch inputs
	_sbi(SW_PORT, SW_RED);
	_sbi(SW_PORT, SW_GREEN);
	_sbi(SW_PORT, SW_YELLOW);
	_sbi(SW_PORT, SW_BLUE);
	
	// setup tone/sound generation
	// set tone pin as output
	_sbi(SW_DDR, TONE_PIN);
	_cbi(SW_PORT, TONE_PIN); 
	
	// set up timer 0 for tone generation
	// toggle OC0A on compare match, mode 2: CTC
	TCCR0A = (0<<COM0A1) | (1<<COM0A0) | (0<<WGM02) |  (1<<WGM01) | (0<<WGM00);
	// prescaler (0 = off, 1 = clk, 2 = 8, 3 = 64, 4 = 256, 5 = 1024) (CS02..CS00)
	TCCR0B = 0; // start with sound off
	
	// set up timer 1 for timeout counting
	// prescaler = 256, mode 4: CTC, normal port operation (OC1A/OC1B disconnected)
	TCCR1A = (0<<WGM11) | (0<<WGM10); 
	TCCR1B = (1<<CS12) | (0<<CS11) | (0<<CS10) |  (0<<WGM13) | (1<<WGM12); 
	OCR1A = 31250; // count up to TOP, interrupt every 1 second @ 8MHz system clock
	// note: the actual interrupt for timer 1 isn't enabled until during game play
	
	// set up PCINT (pin change interrupt)
	PCMSK = 0x1B; // enable PCINT on PCINT0, 1, 2, 4
	GIMSK = (1<<PCIE); // enable pin change interrupt

	sei(); // enable global interrupts now that we're ready to play the game
}


// homebrew pseudo random number generator using linear feedback shift registers
// to save code space instead of using rand() from AVR library
// must be seeded with random_byte before use
// see http://www.maxim-ic.com/app-notes/index.mvp/id/4400 for more details on LFSRs
unsigned char rand(void)
{
	unsigned char sum;
	
	sum = 0;
	
	// This calculates parity on the selected bits (mask = 0xb4).
	if(random_byte & 0x80)
		sum = 1;
	
	if(random_byte & 0x20)
		sum ^= 1;
	
	if(random_byte & 0x10)
		sum ^= 1;
	
	if(random_byte & 0x04)
		sum ^= 1;
	
	random_byte <<= 1;
	random_byte |= sum;
	
	return(random_byte);
}


void delay_ms(unsigned int ms)	// time delay in ms
{
	unsigned int i;
	
	for(i = 0; i < ms; i++)
		_delay_ms(1); // by default, avr gcc inlines _delay_ms. when this code was originally written, 
					  // i created a wrapper to prevent the code size from getting too large
					  // at the time, i didn't know how to force no-inlining, which can be done with __attribute__((noinline)); 
}


void all_led_off(void) // turn off all LEDs
{
	_ledoff(LED_RED);
	_ledoff(LED_GREEN);
	_ledoff(LED_YELLOW);
	_ledoff(LED_BLUE);	
}

/* END OF FILE */

